@@ -6,8 +6,16 @@
 #
 #package PDL::Graphics::TriD::GL;
 
-use PDL::Graphics::OpenGL;
-use PDL::Graphics::OpenGLQ;
+
+BEGIN {
+   use PDL::Config;
+   if ( $PDL::Config{USE_POGL} ) {
+      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(:all)";
+      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
+   } else {
+      eval 'use PDL::Graphics::OpenGL';
+   }
+}
 
 $PDL::Graphics::TriD::create_window_sub = sub {
   return new PDL::Graphics::TriD::GL::Window(@_);
@@ -16,13 +24,13 @@ $PDL::Graphics::TriD::create_window_sub = sub {
 sub PDL::Graphics::TriD::Material::togl{
   my $this = shift;
   my $shin = pack "f*",$this->{Shine};
-  glMaterialfv(&GL_FRONT_AND_BACK,&GL_SHININESS,$shin);
+  glMaterialfv(GL_FRONT_AND_BACK,GL_SHININESS,$shin);
   my $spec = pack "f*",@{$this->{Specular}};
-  glMaterialfv(&GL_FRONT_AND_BACK,&GL_SPECULAR,$spec);
+  glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,$spec);
   my $amb = pack "f*",@{$this->{Ambient}};
   glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,$amb);
   my $diff = pack "f*",@{$this->{Diffuse}};
-  glMaterialfv(&GL_FRONT_AND_BACK,&GL_DIFFUSE,$diff);
+  glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,$diff);
 }
 
 $PDL::Graphics::TriD::any_cannots = 0;
@@ -96,20 +104,20 @@ sub PDL::Graphics::TriD::Object::togl {
 sub PDL::Graphics::TriD::BoundingBox::togl { 
   my($this) = @_;
   $this = $this->{Box};
-  glDisable(&GL_LIGHTING);
+  glDisable(GL_LIGHTING);
   glColor3d(1,1,1);
-  glBegin(&GL_LINES);
+  glBegin(GL_LINES);
   for([0,4,2],[0,1,2],[0,1,5],[0,4,5],[0,4,2],[3,4,2],
 		[3,1,2],[3,1,5],[3,4,5],[3,4,2]) {
 	 &glVertex3d(@{$this}[@$_]);
   }
   glEnd();
-  glBegin(&GL_LINE_STRIP);
+  glBegin(GL_LINE_STRIP);
   for([0,1,2],[3,1,2],[0,1,5],[3,1,5],[0,4,5],[3,4,5]) {
 	 &glVertex3d(@{$this}[@$_]);
   }
   glEnd();
-  glEnable(&GL_LIGHTING);
+  glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::Graph::togl {
@@ -162,11 +170,11 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 
 	$tverts = $this->transform($tverts,$verts,[0,1,2]);
 
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	glColor3d(1,1,1);
 	for(my $j=0;$j<$tverts->getdim(2)-1;$j++){
 	  my $j1=$j+1;
-	  glBegin(&GL_LINES);
+	  glBegin(GL_LINES);
 	  for(my $i=0;$i<$tverts->getdim(1)-1;$i++){
 	    my $i1=$i+1;
 
@@ -187,7 +195,7 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 	  glEnd();
 	}
 
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
 }
 
 
@@ -196,13 +204,14 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 	my($this,$graph) = @_;
 
+        print "togl_axis: got object type " . ref($this) . "\n" if $PDL::debug_trid;
 #	print "TOGLAX\n";
 	my $fontbase = $PDL::Graphics::TriD::GL::fontbase;
 #	print "TOGL EUCLID\n";
    glLineWidth(1); # ought to be user defined
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	glColor3d(1,1,1);
-	glBegin(&GL_LINES);
+	glBegin(GL_LINES);
 	my $dim;
 	for $dim (0..2) {
 		glVertex3f(0,0,0);
@@ -223,9 +232,16 @@ sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 		my $nc = $s->[0];
 		for(0..$ndiv) {
 			&glRasterPos3f(@coords);
-			PDL::Graphics::OpenGL::glpPrintString($fontbase,
-				sprintf("%.3f",$nc));
-			glBegin(&GL_LINES);
+                        if ( $PDL::Config{USE_POGL} ) {
+                              if ( OpenGL::done_glutInit() ) {
+                                 OpenGL::glutBitmapString($fontbase, sprintf("%.3f",$nc));
+                              } else {
+                                 OpenGL::glpPrintString($fontbase, sprintf("%.3f",$nc));
+                              }
+                           } else {
+                              PDL::Graphics::OpenGL::glpPrintString($fontbase, sprintf("%.3f",$nc));
+                           }
+			glBegin(GL_LINES);
 			&glVertex3f(@coords0);
 			&glVertex3f(@coords);
 			glEnd();
@@ -236,10 +252,17 @@ sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 		}
 		$coords0[$dim] = 1.1;
 		&glRasterPos3f(@coords0);
-		PDL::Graphics::OpenGL::glpPrintString($fontbase,
-			$this->{Names}[$dim]);
-	}
-	glEnable(&GL_LIGHTING);
+                if ( $PDL::Config{USE_POGL} ) {
+                   if ( OpenGL::done_glutInit() ) {
+                      OpenGL::glutBitmapString($fontbase, $this->{Names}[$dim]);
+                   } else {
+                      OpenGL::glpPrintString($fontbase, $this->{Names}[$dim]);
+                   }
+                } else {
+                   PDL::Graphics::OpenGL::glpPrintString($fontbase, $this->{Names}[$dim]);
+                }
+             }
+	glEnable(GL_LIGHTING);
 }
 
 
@@ -275,19 +298,37 @@ sub PDL::Graphics::TriD::Points::gdraw {
 #	print "DRAWPOINTS: \n",$points;
 
 	$this->glOptions();
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	PDL::gl_points($points,$this->{Colors});
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
+}
+
+sub PDL::gl_spheres { 
+   my ($coords,$colors) = @_;   
+   for (my $np=0; $np<$coords->dim(1); $np++) {
+      glPushMatrix();
+      my ($x,$y,$z) = ($coords->slice(":,($np)"))->float->list;
+      glTranslatef($x,$y,$z);
+      glutSolidSphere(0.025,15,15);
+      glPopMatrix();
+   }
+}
+
+sub PDL::Graphics::TriD::Spheres::gdraw {
+   my($this,$points) = @_;
+   $this->glOptions();
+   glShadeModel(GL_SMOOTH);
+   PDL::gl_spheres($points,$this->{Colors});
 }
 
 sub PDL::Graphics::TriD::Lattice::gdraw {
 	my($this,$points) = @_;
 
 	$this->glOptions();
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	PDL::gl_line_strip($points,$this->{Colors});
 	PDL::gl_line_strip($points->xchg(1,2),$this->{Colors}->xchg(1,2));
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
 }
 
 
@@ -295,18 +336,18 @@ sub PDL::Graphics::TriD::LineStrip::gdraw {
 	my($this,$points) = @_;
 
 	$this->glOptions();
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	PDL::gl_line_strip($points,$this->{Colors});
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::Lines::gdraw {
 	my($this,$points) = @_;
 
 	$this->glOptions();
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	PDL::gl_lines($points,$this->{Colors});
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::GObject::glOptions {
@@ -336,7 +377,7 @@ sub PDL::Graphics::TriD::Contours::gdraw {
 
   $this->glOptions();
 
-  glDisable(&GL_LIGHTING);
+  glDisable(GL_LIGHTING);
   my $pcnt=0;
   my $i=0;
 
@@ -361,7 +402,7 @@ sub PDL::Graphics::TriD::Contours::gdraw {
 												 ,$this->{LabelStrings});
   }
   
-  glEnable(&GL_LIGHTING);
+  glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::SLattice::gdraw {
@@ -369,10 +410,10 @@ sub PDL::Graphics::TriD::SLattice::gdraw {
 
 	$this->glOptions();
 
-	glPushAttrib(&GL_LIGHTING_BIT | &GL_ENABLE_BIT);
-	glDisable(&GL_LIGHTING);
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (&GL_SMOOTH);
+	glShadeModel (GL_SMOOTH);
 	my @sls1 = (":,0:-2,0:-2",
 	            ":,1:-1,0:-2",
 		    ":,0:-2,1:-1");
@@ -401,10 +442,10 @@ sub PDL::Graphics::TriD::SCLattice::gdraw {
 
 	$this->glOptions();
 
-	glPushAttrib(&GL_LIGHTING_BIT | &GL_ENABLE_BIT);
-	glDisable(&GL_LIGHTING);
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (&GL_FLAT);
+	glShadeModel (GL_FLAT);
 	my @sls1 = (":,0:-2,0:-2",
 	            ":,1:-1,0:-2",
 		    ":,0:-2,1:-1");
@@ -434,11 +475,11 @@ sub PDL::Graphics::TriD::SLattice_S::gdraw {
 
 	$this->glOptions();
 
-	glPushAttrib(&GL_LIGHTING_BIT | &GL_ENABLE_BIT);
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
 # For some reason, we need to set this here as well.
-	glLightModeli(&GL_LIGHT_MODEL_TWO_SIDE, &GL_TRUE);
+	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (&GL_SMOOTH);
+	glShadeModel (GL_SMOOTH);
 	my @sls1 = (":,0:-2,0:-2",
 	            ":,1:-1,0:-2",
 		    ":,0:-2,1:-1");
@@ -474,7 +515,7 @@ sub PDL::Graphics::TriD::SLattice_S::gdraw {
 			      (map {$this->{Colors}->slice($_)} @sls2)
 			     );
 	}
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	if ($this->{Options}{Lines}) {
 	  my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
 	  PDL::gl_line_strip($points,$black);
@@ -489,7 +530,7 @@ sub PDL::Graphics::TriD::SLattice_S::gdraw {
 #
 
 sub PDL::Graphics::TriD::Image::togl {
-#  glDisable(&GL_LIGHTING);
+#  glDisable(GL_LIGHTING);
 #
 # A special construct which always faces the display and takes the entire window
 #  
@@ -512,25 +553,20 @@ sub PDL::Graphics::TriD::Image::gdraw {
 	my($this,$vert) = @_;
 	my ($p,$xd,$yd,$txd,$tyd) = $this->flatten(1); # do binary alignment
 	glColor3d(1,1,1);
-	glTexImage2D(&GL_TEXTURE_2D,
-		0,
-		&GL_RGB,
-		$txd,
-		$tyd,
-		0,
-		&GL_RGB,
-		&GL_FLOAT,
-		${$p->get_dataref()}
-	);
+        if ( $PDL::Config{USE_POGL} ) {
+           glTexImage2D_s(GL_TEXTURE_2D, 0, GL_RGB, $txd, $tyd, 0, GL_RGB, GL_FLOAT, $p->get_dataref());
+        } else {
+           glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, $txd, $tyd, 0, GL_RGB, GL_FLOAT, $p->get_dataref());
+        }
 	 glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
 	    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
 	       glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 	          glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	glNormal3d(0,0,1);
-	glEnable(&GL_TEXTURE_2D);
-	glBegin(&GL_QUADS);
+	glEnable(GL_TEXTURE_2D);
+	glBegin(GL_QUADS);
 	my @texvert = (
 		[0,0],
 		[$xd/$txd, 0],
@@ -543,8 +579,8 @@ sub PDL::Graphics::TriD::Image::gdraw {
 		&glVertex3f($vert->slice(":,($_)")->list);
 	}
 	glEnd();
-	glEnable(&GL_LIGHTING);
-	glDisable(&GL_TEXTURE_2D);
+	glEnable(GL_LIGHTING);
+	glDisable(GL_TEXTURE_2D);
 }
 
 sub PDL::Graphics::TriD::SimpleController::togl {
@@ -564,7 +600,17 @@ sub PDL::Graphics::TriD::SimpleController::togl {
 #
 #
 package PDL::Graphics::TriD::Window;
-use PDL::Graphics::OpenGL;
+
+BEGIN {
+   use PDL::Config;
+   if ( $PDL::Config{USE_POGL} ) {
+      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(:all)";
+      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
+   } else {
+      eval 'use PDL::Graphics::OpenGL';
+   }
+}
+
 use base qw/PDL::Graphics::TriD::Object/;
 use fields qw/Ev Width Height Interactive _GLObject 
               _ViewPorts _CurrentViewPort /;
@@ -581,15 +627,15 @@ sub gdriver {
 	 print "WARNING: Graphics Driver already defined for this window \n";
 	 return;
   }
-  my @db = &GLX_DOUBLEBUFFER;
+  my @db = GLX_DOUBLEBUFFER;
 
   if($PDL::Graphics::TriD::offline) {$options->{x} = -1; @db=()}
 
-  $options->{attributes} = [&GLX_RGBA, @db,
-			    &GLX_RED_SIZE,1,
-			    &GLX_GREEN_SIZE,1,
-			    &GLX_BLUE_SIZE,1,
-			    &GLX_DEPTH_SIZE,1,
+  $options->{attributes} = [GLX_RGBA, @db,
+			    GLX_RED_SIZE,1,
+			    GLX_GREEN_SIZE,1,
+			    GLX_BLUE_SIZE,1,
+			    GLX_DEPTH_SIZE,1,
 			    # Alpha size?
 			   ] unless defined $options->{attributes};
     
@@ -601,30 +647,48 @@ sub gdriver {
 
   print "STARTING OPENGL $options->{width} $options->{height}\n" if($PDL::Graphics::TriD::verbose);
 
-  print "gdriver: Calling OpengGL::OO($options)...\n" if($PDL::debug_trid);
+  print "gdriver: Calling OpengGL::OO($options)...\n" if ($PDL::debug_trid);
 
   $this->{_GLObject}= new PDL::Graphics::OpenGL::OO($options);
 
+  if (exists $this->{_GLObject}->{glutwindow}) {
+     if ($PDL::debug_trid) {
+        print "gdriver: Got OpenGL::OO object(GLUT window ID# " . $this->{_GLObject}->{glutwindow} . ")\n";
+     }
+     $this->{_GLObject}->{winobjects}->[$this->{_GLObject}->{glutwindow}] = $this;      # circular ref
+  }
+
 #glpOpenWindow(%$options);
   
-  print "gdriver: Calling glClearColor...\n" if($PDL::debug_trid);
+  print "gdriver: Calling glClearColor...\n" if ($PDL::debug_trid);
   glClearColor(0,0,0,1);
 
-  print "gdriver: Calling glpRasterFont...\n" if($PDL::debug_trid);
-  my $lb =  $this->{_GLObject}->glpRasterFont(
-						($ENV{PDL_3D_FONT} or "5x8"),0,256);
-  $PDL::Graphics::TriD::GL::fontbase = $lb;
-  #	glDisable(&GL_DITHER);
-  glShadeModel (&GL_FLAT);
-  glEnable(&GL_DEPTH_TEST);
-  glEnable(&GL_NORMALIZE);
-  glEnable(&GL_LIGHTING);
-  glEnable(&GL_LIGHT0);
-  glLightModeli(&GL_LIGHT_MODEL_TWO_SIDE, &GL_TRUE);
+  print "gdriver: Calling glpRasterFont...\n" if ($PDL::debug_trid);
+  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {
+     print STDERR "gdriver: window_type => 'glut' so not actually setting the rasterfont\n" if ($PDL::debug_trid);
+     eval '$PDL::Graphics::TriD::GL::fontbase = GLUT_BITMAP_8_BY_13';
+  } else {
+     # NOTE: glpRasterFont() will die() if the requested font cannot be found
+     #       The new POGL+GLUT TriD implementation uses the builtin GLUT defined
+     #       fonts and does not have this failure mode.
+     
+     my $lb =  eval { $this->{_GLObject}->glpRasterFont( ($ENV{PDL_3D_FONT} or "5x8"), 0, 256 ) };
+     if ( $@ ) {
+        die "glpRasterFont: unable to load font '%s', please set PDL_3D_FONT to an existing X11 font.";
+     }
+     $PDL::Graphics::TriD::GL::fontbase = $lb
+  }
+  #	glDisable(GL_DITHER);
+  glShadeModel (GL_FLAT);
+  glEnable(GL_DEPTH_TEST);
+  glEnable(GL_NORMALIZE);
+  glEnable(GL_LIGHTING);
+  glEnable(GL_LIGHT0);
+  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
   
   # Will this bring us trouble?
   #	if(defined *PDL::Graphics::TriD::GL::Window::glPolygonOffsetEXT{CODE}) {
-#  glEnable(&GL_POLYGON_OFFSET_EXT);
+#  glEnable(GL_POLYGON_OFFSET_EXT);
 #  glPolygonOffsetEXT(0.0000000000001,0.000002);
   #	}
   
@@ -633,7 +697,11 @@ sub gdriver {
 #  $this->reshape($options->{width},$options->{height});
 
   my $light = pack "f*",1.0,1.0,1.0,0.0;
-  glLightfv(&GL_LIGHT0,&GL_POSITION,$light);
+  if ( $PDL::Config{USE_POGL} ) {
+     glLightfv_s(GL_LIGHT0,GL_POSITION,$light);
+  } else {
+     glLightfv(GL_LIGHT0,GL_POSITION,$light);
+  }
 
   glColor3f(1,1,1);
   
@@ -648,8 +716,8 @@ sub gdriver {
 }
 
 sub ev_defaults{
-  return {	&ConfigureNotify => \&doconfig,
-				&MotionNotify => \&domotion,
+  return {	ConfigureNotify => \&doconfig,
+				MotionNotify => \&domotion,
 			}
 }
 
@@ -689,8 +757,10 @@ sub twiddle {
 	 wpic($this->read_picture(),"PDL_$PDL::Graphics::TriD::offlineindex.jpg");
 	 return;
   }
-  if($getout and $dontshow) {
-	 if(!$this->{_GLObject}->XPending()) {return}
+  if ($getout and $dontshow) {
+	 if ( !$this->{_GLObject}->XPending() ) {
+            return;
+         }
   }
   if(!defined $getout) {
 	 $getout = not $PDL::Graphics::TriD::keeptwiddling;
@@ -702,20 +772,26 @@ sub twiddle {
 	 my $hap = 0;
 	 my $gotev = 0;
 
-	 if($this->{_GLObject}->XPending() or !$getout) {
-		@e = $this->{_GLObject}->glpXNextEvent();
-		$gotev=1;
-	 }
+         # Run a MainLoop event if GLUT windows
+         # this pumps the system allowing callbacks to populate
+         # the fake XEvent queue.
+         #
+         glutMainLoopEvent() if $this->{_GLObject}->{window_type} eq 'glut' and not $this->{_GLObject}->XPending();
+
+         if ($this->{_GLObject}->XPending() or !$getout) {
+            @e = $this->{_GLObject}->glpXNextEvent();
+            $gotev=1;
+         }
    print "e= ".join(",",@e)."\n" if($PDL::Graphics::TriD::verbose);
 	
 	 if(@e){
-		if($e[0] == &VisibilityNotify or $e[0] == &Expose) {
+		if ($e[0] == VisibilityNotify || $e[0] == Expose) {
 		  $hap = 1;
-		}elsif($e[0] == &ConfigureNotify) {
+		} elsif ($e[0] == ConfigureNotify) {
 		  print "CONFIGNOTIFE\n" if($PDL::Graphics::TriD::verbose);
 		  $this->reshape($e[1],$e[2]);
 		  $hap=1;
-		}elsif($e[0] == &KeyPress) {
+		} elsif($e[0] == KeyPress) {
 		  print "KEYPRESS: '$e[1]'\n" if($PDL::Graphics::TriD::verbose);
 		  if((lc $e[1]) eq "q") {
 			 $quit = 1;
@@ -773,6 +849,14 @@ sub domotion {
 sub display {
   my($this) = @_;
 
+  return unless defined($this);
+
+  # set GLUT context to current window (for multiwindow support
+  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {
+     glutSetWindow($this->{_GLObject}->{glutwindow});
+  }
+
+  print "display: calling glClear()\n" if ($PDL::Graphics::TriD::verbose);
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   glMatrixMode(GL_MODELVIEW);
 
@@ -786,7 +870,7 @@ sub display {
 
 
 	 if($vp->{Transformer}) {
-		print "Transforming viewport!\n" if($PDL::Graphics::TriD::verbose);
+		print "display: transforming viewport!\n" if ($PDL::Graphics::TriD::verbose);
 		$vp->{Transformer}->togl();
 	 }
 
@@ -799,7 +883,20 @@ sub display {
 
   }
 
-  $this->{_GLObject}->glXSwapBuffers();
+  if ( $PDL::Config{USE_POGL} ) {
+
+     print "display: SwapBuffers() call on return\n" if ($PDL::Graphics::TriD::verbose);
+     if ( $this->{_GLObject}->{window_type} eq 'glut' ) {  # need to make method call
+        glutSwapBuffers();
+     } elsif ( $this->{_GLObject}->{window_type} eq 'x11' ) {  # need to make method call
+        $this->{_GLObject}->glXSwapBuffers();
+     } else {
+        print "display: got object with inconsistent _GLObject info\n";
+     }
+
+  } else {
+     $this->{_GLObject}->glXSwapBuffers();
+  }
 #  $this->{Angle}+= 3;
 }
 
@@ -808,10 +905,15 @@ sub read_picture {
 	my($this) = @_;
 	my($w,$h) = @{$this}{qw/Width Height/};
 	my $res = PDL->zeroes(PDL::byte,3,$w,$h);
-	glPixelStorei(&GL_UNPACK_ALIGNMENT,1);
-	glPixelStorei(&GL_PACK_ALIGNMENT,1);
-	glReadPixels(0,0,$w,$h,&GL_RGB,&GL_UNSIGNED_BYTE,
-		${$res->get_dataref});
+	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
+	glPixelStorei(GL_PACK_ALIGNMENT,1);
+
+        if ( $PDL::Config{USE_POGL} ) {
+           glReadPixels_s(0,0,$w,$h,GL_RGB,GL_UNSIGNED_BYTE,$res->get_dataref);
+        } else {
+           glReadPixels(0,0,$w,$h,GL_RGB,GL_UNSIGNED_BYTE,$res->get_dataref);
+        }
+
 	return $res;
 }
 
@@ -820,29 +922,39 @@ sub read_picture {
 # EVENT HANDLER MINIPACKAGE FOLLOWS!
 
 package PDL::Graphics::TriD::EventHandler;
-use PDL::Graphics::OpenGL qw/ConfigureNotify MotionNotify ButtonPress ButtonRelease 
-                             Button1Mask Button2Mask Button3Mask/;
+
+BEGIN {
+   use PDL::Config;
+   if ( $PDL::Config{USE_POGL} ) {
+      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(ConfigureNotify MotionNotify ButtonPress ButtonRelease Button1Mask Button2Mask Button3Mask)";
+      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
+   } else {
+      eval 'use PDL::Graphics::OpenGL';
+   }
+}
+
 use fields qw/X Y Buttons VP/;
 use strict;
 sub new {
   my $class = shift;
   my $vp = shift;
   no strict 'refs';
-  my $self = bless [ \%{"$class\::FIELDS"}], $class;
+  my $self = fields::new($class);
   $self->{X} = -1;
   $self->{Y} = -1;
   $self->{Buttons} = [];
   $self->{VP} = $vp;
+
   $self;
 }
 
 sub event {
   my($this,$type,@args) = @_;
 
-    print "EH: ",ref($this)," $type (",join(",",@args),")\n" if($PDL::Graphics::TriD::verbose);
+  print "EH: ",ref($this)," $type (",join(",",@args),")\n" if($PDL::Graphics::TriD::verbose);
   my $retval;
 
-  if($type == &MotionNotify) {
+  if($type == MotionNotify) {
 	 my $but = -1;
 	 
   SWITCH: { 
@@ -865,7 +977,7 @@ sub event {
 	 $this->{X} = $args[1]; $this->{Y} = $args[2];
   NOBUT:
 
-       } elsif($type == &ButtonPress) {
+       } elsif($type == ButtonPress) {
 
 	 my $but = $args[0]-1;
 	 print "BUTTONPRESS $but\n" if($PDL::Graphics::TriD::verbose);
@@ -873,18 +985,18 @@ sub event {
 	 $retval = $this->{Buttons}[$but]->ButtonPress($args[1],$args[2]) 
 	   if($this->{Buttons}[$but]);
 
-       } elsif($type == &ButtonRelease) {
+       } elsif($type == ButtonRelease) {
 
 	 my $but = $args[0]-1;
 	 print "BUTTONRELEASE $but\n" if($PDL::Graphics::TriD::verbose);
 	 $retval = $this->{Buttons}[$but]->ButtonRelease($args[1],$args[2]) 
 	   if($this->{Buttons}[$but]);
 
-       } elsif($type== &ConfigureNotify) {
+       } elsif($type== ConfigureNotify) {
 
 	 # Kludge to force reshape of the viewport associated with the window -CD
 	 print "ConfigureNotify (".join(",",@args).")\n" if($PDL::Graphics::TriD::verbose);
-	 print "viewport is $this->[4]\n" if($PDL::Graphics::TriD::verbose);
+	 print "viewport is $this->{VP}\n" if($PDL::Graphics::TriD::verbose);
 #	 $retval = $this->reshape(@args);
 
        }
@@ -906,7 +1018,16 @@ use base qw/PDL::Graphics::TriD::Object/;
 use fields qw/X0 Y0 W H Transformer EHandler Active ResizeCommands 
               DefMaterial AspectRatio Graphs/;
 
-use PDL::Graphics::OpenGL;
+BEGIN {
+   use PDL::Config;
+   if ( $PDL::Config{USE_POGL} ) {
+      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(:all)";
+      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
+   } else {
+      eval 'use PDL::Graphics::OpenGL';
+   }
+}
+
 use PDL::Graphics::OpenGLQ;
 
 
@@ -922,7 +1043,7 @@ sub highlight {
 
   $colors = PDL->ones(3,5);
 
-  glDisable(&GL_LIGHTING);
+  glDisable(GL_LIGHTING);
   
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
@@ -936,7 +1057,7 @@ sub highlight {
 
   glLineWidth(1);
 		
-  glEnable(&GL_LIGHTING);
+  glEnable(GL_LIGHTING);
 
 }
 
@@ -949,6 +1070,7 @@ sub do_perspective {
 
 	if($PDL::Graphics::TriD::verbose>1){
 	  my ($i,$package,$filename,$line);
+          $i = 0;
 	  do { 
 	    ($package,$filename,$line) = caller($i++);
 	    print "$package ($filename, line $line)\n";
