@@ -16,7 +16,7 @@ ftp://wuarchive.wustl.edu/graphics/graphics/packages/NetPBM/
 Pbmplus (on which netpbm is based) might work as well, I haven't tried it.
 If you want to read/write JPEG images you additionally need the two
 converters cjpeg/djpeg which come with the libjpeg distribution (the
-"official" archive site for this software is ftp://ftp.uu.net/graphics/jpeg).
+"official" archive site for this software is L<ftp://ftp.uu.net/graphics/jpeg>).
 
 Image I/O for all formats is established by reading and writing only
 the PNM format directly while the netpbm standalone apps take care of
@@ -26,17 +26,14 @@ here for 'portable any map' meaning any of the PBM/PGM/PPM formats.
 As it appeared to be a reasonable place this package also contains the
 routine wmpeg to write mpeg movies from PDLs representing image
 stacks (the image stack is first written as a sequence of PPM images into some
-temporary directory). For this to work you additionally need the program
-mpeg_encode from the Berkeley multimedia package.
-
-You find mpeg_encode at ftp://mm-ftp.cs.berkeley.edu/pub/multimedia/mpeg/encode (they even have binaries for a number of platforms).
+temporary directory). For this to work you need the program ffmpeg also.
 
 =cut
 
 package PDL::IO::Pic;
 
 
-@EXPORT_OK = qw( wmpeg rpic wpic rpiccan wpiccan );
+@EXPORT_OK = qw( wmpeg rim wim rpic wpic rpiccan wpiccan );
 
 %EXPORT_TAGS = (Func => [@EXPORT_OK]);
 use PDL::Core;
@@ -45,8 +42,10 @@ use PDL::Types;
 use PDL::ImageRGB;
 use PDL::IO::Pnm;
 use PDL::Options;
+use PDL::Config;
 use File::Basename;
 use SelfLoader;
+use File::Spec;
 
 use strict;
 use vars qw( $Dflags @ISA %converter );
@@ -60,7 +59,7 @@ The executables from the netpbm package are assumed to be in your path.
 Problems in finding the executables may show up as PNM format
 errors when calling wpic/rpic. If you run into this kind of problem run
 your program with perl C<-w> so that perl prints a message if it can't find
-the filter when trying to open the pipe.
+the filter when trying to open the pipe. [']
 
 =cut
 
@@ -78,11 +77,11 @@ the filter when trying to open the pipe.
 #
 #
 # The "referral" field, if present, contains a within-perl referral
-# to other methods for reading/writing the PDL as that type of file.  The 
+# to other methods for reading/writing the PDL as that type of file.  The
 # methods must have the same syntax as wpic/rpic (e.g. wfits/rfits).
 #
 
-$PDL::IO::Pic::debug = 0;
+$PDL::IO::Pic::debug = $PDL::IO::Pic::debug || 0;
 &init_converter_table();
 
 # setup functions
@@ -91,16 +90,42 @@ sub init_converter_table {
   # default flag to be used with any converter unless overridden with FLAGS
   $Dflags = '-quiet';
   %converter = ();
-  for ('TIFF','SGI','RAST','PCX','PNG')
+
+  # Pbmplus systems have cjpeg/djpeg; netpbm systems have pnmtojpeg and
+  # jpegtopnm.
+
+  my $jpeg_conv='';
+
+  {
+      my @path = File::Spec->path();
+      my $ext = $^O =~ /MSWin/i ? '.exe' : '';
+      local $_;
+      my $pbmplus;
+
+      for (@path) {
+	  $jpeg_conv="cjpeg" if(-x "$_/cjpeg");
+	  $jpeg_conv="pnmtojpeg" if(-x "$_/pnmtojpeg" . $ext);
+      }
+  }
+
+  my @normal = qw/TIFF SGI RAST PCX PNG/;
+  push(@normal,"JPEG") if($jpeg_conv eq 'pnmtojpeg');
+
+  for (@normal)
     { my $conv = lc; $converter{$_} = {put => "pnmto$conv",
 				       get => "$conv".'topnm'} }
-  for (['PNM','NONE','NONE'],
-       ['JPEG','cjpeg','djpeg'],
-       ['PS','pnmtops',
-              'gs -sDEVICE=ppmraw -sOutputFile=- -q -dNOPAUSE -dBATCH'],
 
-       ['GIF','ppmtogif','giftopnm'],
-       ['IFF','ppmtoilbm','ilbmtoppm']) {
+  my @special = (['PNM','NONE','NONE'],
+		 ['PS','pnmtops',
+		  'gs -sDEVICE=ppmraw -sOutputFile=- -q -dNOPAUSE -dBATCH'],
+
+		 ['GIF','ppmtogif','giftopnm'],
+		 ['IFF','ppmtoilbm','ilbmtoppm']
+		 );
+  push(@special,['JPEG', 'cjpeg' ,'djpeg'])
+      if($jpeg_conv eq 'cjpeg');
+
+   for(@special) {
     $converter{$_->[0]} = {put => $_->[1],
 			   get => $_->[2]}
   }
@@ -115,10 +140,10 @@ sub init_converter_table {
   my $key;
   for $key (keys %converter) {
 
-    $converter{$key}->{Rok} = inpath($converter{$key}->{'get'}) 
+    $converter{$key}->{Rok} = inpath($converter{$key}->{'get'})
       if defined($converter{$key}->{'get'});
 
-    $converter{$key}->{Wok} = inpath($converter{$key}->{'put'}) 
+    $converter{$key}->{Wok} = inpath($converter{$key}->{'put'})
       if defined($converter{$key}->{'put'});
 
     if (defined $converter{$key}->{Prefilt}) {
@@ -201,6 +226,7 @@ sub PDL::rpiccan {splice @_,1,0,'R';
 sub PDL::wpiccan {splice @_,1,0,'W';
 		  return PDL::IO::Pic::piccan(@_)}
 
+
 =head2 rpic
 
 =for ref
@@ -476,37 +502,221 @@ sub PDL::wpic {
     wpnm($pdl, $cmd, $iform , $iraw);
 }
 
+=head2 rim
+
+=for usage
+
+  $a = rim($file);
+  rim($a,$file);
+
+=for ref
+
+Read images in most formats, with improved RGB handling.
+
+You specify a filename and get back a PDL with the image data in it.
+Any PNM handled format or FITS will work. In the second form, $a is an
+existing PDL that gets loaded with the image data.
+
+If the image is in one of the standard RGB formats, then you get back
+data in (<X>,<Y>,<RGB-index>) format -- that is to say, the third dim
+contains the color information.  That allows you to do simple indexing
+into the image without knowing whether it is color or not -- if present,
+the RGB information is silently threaded over.  (Contrast L<rpic>, which
+munges the information by putting the RGB index in the 0th dim, screwing
+up subsequent threading operations).
+
+If the image is in FITS format, then you get the data back in exactly
+the same order as in the file itself.
+
+Images with a ".Z" or ".gz" extension are assumed to be compressed with
+UNIX L<"compress"|compress> or L<"gzip"|gzip>, respecetively, and are
+automatically uncompressed before reading.
+
+OPTIONS
+
+The same as L<rpic>, which is used as an engine:
+
+=over 3
+
+=item FORMAT
+
+If you don't specify this then formats are autodetected.  If you do specify
+it then only the specified interpreter is tried.  For example,
+
+  $a = rim("foo.gif",{FORMAT=>"JPEG"})
+
+forces JPEG interpretation.
+
+=item XTRAFLAGS
+
+Contains extra command line flags for the pnm interpreter.  For example,
+
+  $a = rim("foo.jpg",{XTRAFLAGS=>"-nolut"})
+
+prevents use of a lookup table in JPEG images.
+
+=back
+
+=cut
+
+use PDL::IO::Pic;
+
+sub rim {
+  my(@args) = @_;
+
+  if(@args == 2) {
+    my($dest) = $args[0];
+    if($dest->dim(0) == 3) {
+      $args[0] = $dest->reorder(1,2,0);
+    }
+    return rpic(@args);
+  }
+
+  my $out = rpic(@args);
+
+  #
+  # Check for RGB and reorder dims if necessary.  The SIMPLE test is to check
+  # if the image has a FITS header.
+  #
+  # (What a kludge -- but rpic is historical and has to be kept at this point)
+  #
+  if($out->ndims == 3 && $out->dim(0) == 3 &&
+     !( defined($out->gethdr) && $out->gethdr->{SIMPLE} )
+     ) {
+    return $out->reorder(1,2,0);
+  }
+
+  $out;
+}
+
+
+
+=head2 wim
+
+=for usage
+
+  wim $pdl,$file;
+  $pdl->wim("foo.gif",{LUT=>$lut});
+
+=for ref
+
+Write out an image file.  You can specify the format explicitly as an
+option, or the function will try to guess the correct image
+format from the filename extension, e.g.
+
+  $pdl->wim("image.gif");
+  $pdl->wim("image.fits");
+
+will write a gif and a FITS file.  The data written out will be scaled
+to byte if the input if of type float/double.  Input data that is of a
+signed integer type and contains negative numbers will be rejected.
+
+If you append C<.gz> or C<.Z> to the end of the file name, the final
+file will be automatically compresed with L<"gzip"|gzip> |
+L<"compress"|compress>, respectively.
+
+OPTIONS
+
+You can pass in a hash ref whose keys are options.  The code uses the
+PDL::Options module so unique abbreviations are accepted.  Accepted
+keys are the same as for L<wpic|wpic>, which is used as an engine:
+
+=over 3
+
+=item CONVERTER
+
+Names the converter program to be used by pbmplus (e.g. "ppmtogif" to
+output a gif file)
+
+=item FLAGS
+
+Flags that should be passed to the converter (replacing any default flag list)
+e.g. "-interlaced" to make an interlaced GIF
+
+=item IFORM
+
+Explicitly specifies the intermediate format (e.g. PGM, PPM, or PNM).
+
+=item XTRAFLAGS
+
+Flags that should be passed to the converter (in addition to any default
+flag list).
+
+=item FORMAT
+
+Explicitly specifies the output image format (allowing pbmplus to pick an
+output converter)
+
+=item COLOR
+
+Specifies color conversion (e.g. 'bw' converts to black-and-white; see
+L<pbmplus> for details).
+
+=item LUT
+
+Use color-table information
+
+=back
+
+=cut
+
+*wim = \&PDL::wim;
+
+sub PDL::wim {
+  my(@args) = @_;
+
+  my($im) = $args[0];
+
+  $args[0] = $im->reorder(2,0,1)
+    if(    $im->ndims == 3
+       and $im->dim(2)==3
+       and !(
+	      ( $args[1] =~ m/\.fits$/i )
+	      or
+	      ( ref $args[2] eq 'HASH' and $args[2]->{FORMAT} =~ m/fits/i )
+	    )
+       );
+
+  wpic(@args);
+}
+
 =head2 wmpeg
 
 =for ref
 
-Write an image sequence ((x,y,n) piddle) as an MPEG animation.
+Write an image sequence ((3,x,y,n) piddle) as an MPEG animation.
 
 =for example
 
    $anim->wmpeg("GreatAnimation.mpg");
 
-Writes a stack of rgb images as an mpeg movie. Expects a 4-D pdl of type byte
-as input. First dim has to be 3 since it is interpreted as interlaced RGB.
-Some of the input data restrictions will have to be relaxed in the future but
-routine serves as a proof of principle at the moment. It uses the program
-mpeg_encode from the Berkeley multimedia package (see also text at the top of
-this package). Mpeg parameters written by this routines haven't been tweaked
-in any way yet (in other words, lots of room for improvement). For an example
-how to use the routine see appropriate test that comes with this package.
-Currently, wmpeg doesn't allow modification of the parameters written through
-its calling interface. This will change in the future as needed.
+Writes a stack of rgb images as an mpeg movie. Expects
+a 4-D pdl of type byte as input. First dim has to be
+3 since it is interpreted as interlaced RGB. Some of
+the input data restrictions will have to be relaxed in
+the future but routine serves as a proof of principle
+at the moment. It uses the program ffmpeg to encode
+the frames into video. The arguments and parameters
+used for ffmpeg have not been tuned. This is a first
+implementation replacing mpeg_encode by ffmpeg.
+Currently, wmpeg doesn't allow modification of the
+parameters written through its calling interface. This
+will change in the future as needed.
 
-In the future it might be much nicer to implement a movie perl object that
-supplies methods for manipulating the image stack (insert, cut,
-append commands) and a final movie->make() call would invoke mpeg_encode on the
-picture stack (which will only be held on disk). This should get around the
-problem of having to hold a huge amount of data in memory to be passed into
-wmpeg (when you are, e.g. writing a large animation from PDL3D rendered
-fly-throughs). Having said that, the actual storage requirements might not be
-so big in the future any more if you could pass 'virtual' transform pdls into
-wmpeg  that will only be actually calculated when accessed by the wpic
-routines, you know what I mean...
+In the future it might be much nicer to implement
+a movie perl object that supplies methods for
+manipulating the image stack (insert, cut, append
+commands) and a final movie->make() call would invoke
+ffmpeg on the picture stack (which will only be held on
+disk). This should get around the problem of having to
+hold a huge amount of data in memory to be passed into
+wmpeg (when you are, e.g. writing a large animation
+from PDL3D rendered fly-throughs). Having said that,
+the actual storage requirements might not be so big
+in the future any more if you could pass 'virtual'
+transform pdls into wmpeg that will only be actually
+calculated when accessed by the wpic routines, you know
+what I mean...
 
 
 =cut
@@ -514,71 +724,49 @@ routines, you know what I mean...
 *wmpeg = \&PDL::wmpeg;
 
 sub PDL::wmpeg {
-    barf 'Usage: wmpeg($pdl,$filename) ' .
-	'or $pdl->wmpeg($filename)' if $#_ != 1;
+   barf 'Usage: wmpeg($pdl,$filename) ' .
+   'or $pdl->wmpeg($filename)' if $#_ != 1;
 
-    my ($pdl,$file) = @_;
-    my @Dims = $pdl->dims;
-    # too strict in general but alright for the moment
-    # especially restriction to byte will have to be relaxed
-    barf "input must be byte (3,x,y,z)" if (@Dims != 4) || ($Dims[0] != 3)
-	|| ($pdl->get_datatype != $PDL_B);
-    my $nims = $Dims[3];
-    my $tmp = gettmpdir();
-    my $tmpdir = "$tmp/wmpeg$$";
-    barf "directory $tmpdir already exists, clear up first" if -d $tmpdir;
-    mkdir $tmpdir,0700;
-    # check the pdl for correct dimensionality
+   my ($pdl,$file) = @_;
+   my @Dims = $pdl->dims;
+   # too strict in general but alright for the moment
+   # especially restriction to byte will have to be relaxed
+   barf "input must be byte (3,x,y,z)" if (@Dims != 4) || ($Dims[0] != 3)
+   || ($pdl->get_datatype != $PDL_B);
+   my $nims = $Dims[3];
+   my $tmp = gettmpdir();
 
-    # write all the images as ppms and write the appropriate parameter file
-    my ($i,$fname);
-    # add blank cells to each image to fit with 16N x 16N mpeg standard
-    # $frame is full frame, insert each image in as $inset
-    my (@MDims) = (3,map(16*int(($_+15)/16),@Dims[1..2]));
-    my ($frame) = zeroes(@MDims);
-    my ($inset) = $frame->slice(join(',',
-                              map(int(($MDims[$_]-$Dims[$_])/2).':'.
-                                  int(($MDims[$_]+$Dims[$_])/2-1),0..2)));
-    my $range = sprintf "[%d-%d]",0,$nims-1;
-    # write the parameter file
-    open PAR,">$tmpdir/mpeg.params" or barf "can't open mpeg parameter file";
-    print PAR <<"EOT";
-PATTERN		IBBBBBBBBBBP
-OUTPUT		$file
-GOP_SIZE	16
-SLICES_PER_FRAME	5
-BASE_FILE_FORMAT	PPM
-INPUT_CONVERT *
-INPUT_DIR	stdin
-INPUT
-frame.*.ppm $range
-END_INPUT
-PIXEL		FULL
-RANGE		5
-PSEARCH_ALG	LOGARITHMIC
-BSEARCH_ALG	SIMPLE
-IQSCALE		6
-PQSCALE		6
-BQSCALE		6
-REFERENCE_FRAME	ORIGINAL
-FORCE_ENCODE_LAST_FRAME
-EOT
-    close PAR;
-    open MPEG, "| mpeg_encode $tmpdir/mpeg.params" or
-          barf "spawning mpeg_encode failed: $?";
-    binmode MPEG;
-    my (@slices) = $pdl->dog;
-    for ($i=0; $i<$nims; $i++) {
-      print STDERR "Writing frame $i\n";
+   # get tmpdir for parameter file
+   # see PDL-2.4.6 version for original code
+
+   # check the pdl for correct dimensionality
+
+   # write all the images as ppms and write the appropriate parameter file
+   my ($i,$fname);
+   # add blank cells to each image to fit with 16N x 16N mpeg standard
+   # $frame is full frame, insert each image in as $inset
+   my (@MDims) = (3,map(16*int(($_+15)/16),@Dims[1..2]));
+   my ($frame) = zeroes(byte,@MDims);
+   my ($inset) = $frame->slice(join(',',
+         map(int(($MDims[$_]-$Dims[$_])/2).':'.
+            int(($MDims[$_]+$Dims[$_])/2-1),0..2)));
+   my $range = sprintf "[%d-%d]",0,$nims-1;
+   open MPEG, "| ffmpeg -f image2pipe -vcodec ppm -i -  $file"
+      or barf "spawning ffmpeg failed: $?";
+   binmode MPEG;
+   # select ((select (MPEG), $| = 1)[0]);  # may need for win32
+   my (@slices) = $pdl->dog;
+   for ($i=0; $i<$nims; $i++) {
+      local $PDL::debug = 1;
+      print STDERR "Writing frame $i, " . $frame->slice(':,:,-1:0')->clump(2)->info . "\n";
       $inset .= $slices[$i];
       print MPEG "P6\n$MDims[1] $MDims[2]\n255\n";
-      pnmout($frame->slice(':,:,-1:0')->clump(2),
-             1, 0, 'PDL::IO::Pic::MPEG');
-    }
-    # clean up
-    close MPEG;
-    unlink <$tmpdir/*>;
-    rmdir $tmpdir or barf "couldn't delete temporary dir $tmpdir";
+      pnmout($frame->slice(':,:,-1:0')->clump(2), 1, 0, 'PDL::IO::Pic::MPEG');
+   }
+   # clean up
+   close MPEG;
+
+   # rm tmpdir and files if needed
 }
 
 
@@ -762,15 +950,15 @@ sub chkpdl {
     return ($pdl, $iform);
 }
 
+# delegate setting the temporary directory to the config file
+# (so that it can either be OS-independent or at least
+#  easily controlled by the user).
+#
 sub gettmpdir {
-    # in the future an os indep. way
-    # are there already VMS or Windows NT/95 users ?
-  my $tmpdir = '/tmp';
-  $tmpdir  = $ENV{"TMP"} if defined $ENV{"TMP"};
-  $tmpdir  = $ENV{"TEMP"} if defined $ENV{"TEMP"};
-  $tmpdir =~ s|/$||;  # chop off a trailing '/'
-  barf "can't locate a temp dir" unless -d $tmpdir;
-  return $tmpdir;
+    my $tmpdir = $PDL::Config{TEMPDIR} ||
+      die "TEMPDIR not found in %PDL::Config";
+    barf "can't locate a temp dir called $tmpdir" unless -d $tmpdir;
+    return $tmpdir;
 }
 
 
